#! /usr/bin/env perl

# use strict;

# remember errors and return with 10 if trace was not okay
my $error = 0;

# remember communicator names
my %CommNames = ();

# enumerate activities and map from old IDs to new ones and texts
my $NumActs = 1;
my %DefActs = ();

# enumerate OpenMP construcst and map from old IDs to new ones and texts
my $NumOpenMP = 1;
my %DefOpenMP = ();

# enumerate states and map to old ID and name
my $NumStates = 1;
my %MapStates = ();
my %DefStates = ();
my %DefStatesAct = ();

# enumerate scopes and map to old ID and name
my $NumScopes = 1;
my %MapScopes = ();
my %DefScopes = ();
my %DefScopesSCLs = ();
my %DefScopesAct = ();

# enumerate global operations and map to their names
my $NumOps = 1;
my %MapOps = ();
my %DefOps = ();

# enumerate files and map to their names
my $NumFiles = 1;
my %MapFiles = ();
my %DefFiles = ();

# enumerate source code and map them to their (list of) location(s)
my $NumSrcs = 1;
my %MapSrcs = ();
my %DefSrcs = ();

# map from number to name
my %DefGroups = ();
my %DefCounters = ();
my %DefCounterClasses = ();
my %DefParreg = ();

# map from token to name and communicator
my %DefIO = ();

# map from CPU number to name
my %DefCPUNames = ();

sub AddGroupBits {
  my $groups = shift( @_ );
  $groups =~ s/(\d+)/ $1 | 0x80000000 /eg;
  return $groups;
}

sub TranslateCPUs {
  my $CPUs = shift( @_ );
  $CPUs =~ s/(^|\s?)(\d+)(\s?|$)/ "$1" . ( ($2 & 0x80000000) ? ( defined($DefGroups{$2}) ?
							       ("\"$DefGroups{$2}\"") :
							       ( ($error = 1, print STDERR "undefined group $2"), "$2" ) ) :
					  ((int($2 \/ 0x10000 ) . ":" . int($2 % 0x10000))) ) . "$3"/eg;
  return $CPUs;
}

for( $i=0; $i<=$#ARGV; $i++ ) {
    if( defined( $ARGV[$i] ) && ( $ARGV[$i] =~ /\-+help/ || $ARGV[$i] eq "\-h" ) ) {
	die "This tool expands an ASCII-dump of stftool so that tokens (integers)\n"
	    ."are replaced by their names (strings).\n"
	    ."Simply pipe the output into it or give the files as its arguments\n"
	    ."and the expanded trace is dumped to stdout.\n";
    }
}

while (<>) {
  s/\r//g; # needed for Windows

  # fix missing group bits
  s/^DEFGROUP\s+(\d+)/"DEFGROUP " . AddGroupBits( "$1" )/eg;
  s/^DEFGROUP(.*?)\bGROUPS\s+((\d+\s+)+)/"DEFGROUP$1" . "GROUPS " . AddGroupBits( "$2" )/eg;
  s/^DEFSAMP(.*?)\bGROUP\s+(\d+)/"DEFSAMP$1" . "GROUP " . AddGroupBits( "$2" )/eg;
  # fix old format of DEFSAMP record
  s/^DEFSAMP\s+(\d+)\s+CLASS\s+(\d+)\s+(CPU|GROUP)\s+(\d+)/DEFSAMP $1 CLASS $2 CPUSAMP FOR $4/g;
  # fix old format of DEFGROUP record
  s/^DEFGROUP(.*?)\bNCPUS\s+\d+\s+CPUS\s+((\d+\s*)+)/DEFGROUP$1$2/g;
  s/^DEFGROUP(.*?)\bNGROUPS\s+\d+\s+GROUPS\s+((\d+\s*)+)/DEFGROUP$1$2/g; 

  s/\sSCL\s+268435453/ SCL none/g;

  if ( /^COMDEF\s+(\d+).*?(?:NAME\s+("[^"]*?"|\w+))?\n/ ) {
    my $name = defined $2 ? $2 : "";
    my $commid = $1;

    $name =~ s/"(.*)"/$1/;
    $name =~ s/\s+/_/g;
    $CommNames{$commid} = $name;
  } elsif (s/^DEFACT\s+(\d+)\s+"?([^"]*)"?/DEFACT $NumActs "$2"/g) {
    $DefActs{$1} = $2;
    $NumActs++;
  } elsif (s/^DEFOPENMP\s+ID\s+(\d+)\s+NAME\s+"?([^"]*)"?/DEFOPENMP "$2"/g) {
    $DefOpenMP{$1} = $2;
    $NumOpenMP++;
  } elsif (s/^DEFSTATE\s+(\d+)\s+ACT\s+(\d+)\s+"?([^"]*)"?/DEFSTATE $NumStates ACT "$DefActs{$2}:$3"/g) {
    $MapStates{$1} = $NumStates;
    $DefStates{$1} = $3;
    $DefStatesAct{$1} = $2;
    $NumStates++;
  } elsif (s/^DEFSCOPE\s+(\d+)\s+ACT\s+(\d+)\s+"?([^"]*)"?(\s+START SCL \d+|[^\n]*)(\s+END SCL \d+|[^\n]*)/DEFSCOPE $NumStates ACT "$DefActs{$2}:$3"$4$5/g) {
    $MapScopes{$1} = $NumScopes;
    $DefScopes{$1} = $3;
    $DefScopesSCLs{$1} = "$4$5";
    $DefScopesAct{$1} = $2;
    $NumScopes++;
  } elsif (s/^GLOBALOPTOKEN\s+(\d+)\s+"?([^"]*)\"?/GLOBALOPTOKEN $NumOps "$2"/g) {
    $MapOps{$1} = $NumOps;
    $DefOps{$1} = $2;
    $NumOps++;
  } elsif (s/^FILETOKEN\s+(\d+)\s+"?([^"]*)\"?/FILEOPTOKEN $NumFiles "$2"/g) {
    $MapFiles{$1} = $NumFiles;
    $DefFiles{$1} = $2;
    $NumFiles++;
  } elsif (s/^DEFSCL\s+(\d+)(?:\s+NLOCS)?\s+(\d+)\s+(.*)/DEFSCL $NumSrcs NLOCS $2 $3/g) {
    $MapSrcs{$1} = $NumSrcs;
    $DefSrcs{$1} = $3;
    $DefSrcs{$1} =~ s/LOCS\s*//;
    $NumSrcs++;
  } elsif (s/^DEFSAMPCLASS\s+(\d+)\s+"?([^"]*)\"?/DEFSAMPCLASS "$2"/g) {
    $DefCounterClasses{$1} = $2;
  } elsif (s/^DEFGROUP\s+(\d+)\s+"?([^"]*)\"?\s+NMEMBS\s+(\d+)\s+(.*)/"DEFGROUP $2 NMEMBS $3 " . TranslateCPUs( $4 )/eg) {
    $DefGroups{$1} = $2;
  } elsif (s/^DEFKPARREG\s+(\d+)\s+"?([^"]*)\"?/DEFKPARREG "$2"/g) {
    $DefParreg{$1} = $2;
  } elsif (s/^DEFSAMP\s+(\d+)\s+CLASS\s+(\d+)\s+(\S+)\s+FOR\s+(\d+)(.*)\sNAME\s+\"?([^"]*)\"?/
    "DEFSAMP \"$6\" CLASS " .
      ( defined( $DefCounterClasses{$2} ) ? "\"$DefCounterClasses{$2}\"" : ( ($error = 1, print STDERR "undefined class $2: $_"), $2 ) ) .
      " $3 FOR " .
      ( defined( $DefGroups{$4} ) ? "\"$DefGroups{$4}\"" : ( ($error = 1, print STDERR "undefined group $4: $_"), $4 ) ) .
      "$5"/eg ) {
    $DefCounters{$1} = "$DefCounterClasses{$2}:$6";
    # perl bug?! $4 contains "CLASS <number>"
    s/4:\s+CLASS\s+\d+\s+/ /;
  } elsif (s/^CPUSYM\s+(\d+)\s+(.*)/"CPUSYM " . TranslateCPUs( $1 ) . " $2"/e) {
    $DefCPUNames{$1} = $2;
  } elsif (/^IOFILE\s+(\d+)\s+(\d+)\s+(.*)\n/) {
    if (defined $DefIO{$1}) {
      $error = 1;
      print STDERR "IOFILE token used more than once: $_";
    } else {
      $DefIO{$1} = "COM $2 $3";
    }
    s/IOFILE\s+(\d+)\s+(\d+)\s+(.*)/IOFILE $1 COM $2 $3/;
    if( /\s+COM\s+(\d+)\b/ ) {
      defined $CommNames{$1} ?
	s/\s+COM\s+(\d+)\b/ COM $CommNames{$1}($1)/g :
	($error = 1, print (STDERR "undefined communicator $1\n") );
    }
  } else {
    ( s/\bKPARREG(BEGIN|END|BARSUM)\s+(\d+)\b/ "KPARREG$1 " . (($1 eq "BARSUM" && $2 == 1) ? "OPAQUE" : defined($DefParreg{$2}) ? "\"$DefParreg{$2}\"" : ($error = 1, (print STDERR "Undefined parallel region $2: $_"), $2 ) )/e
    ) or( /\bSAMP\b/ and (s/\bDEF\s+(\d+)/"DEF " . (defined($DefCounters{$1}) ? "\"$DefCounters{$1}\"" : ( ($error = 1, print STDERR "Undefined counter $1: $_") , $1 ) )/eg, 1 )
    ) or
    ( s/OPENMPENTER\s+CPU\s+(\d+)\s+OP\s+(\d+)\s+TAG\s+"?([^"]*)"?/
      "OPENMPENTER CPU $1 OP " .
      (defined($DefOpenMP{$2}) ? "\"$DefOpenMP{$2}\"" :
       ($error = 1, (print STDERR "Undefined OpenMP $2: $_"), $2 ) ) .
      " TAG \"$3\"" /e ) or
    ( s/PARREGBEGIN\s+CPU\s+(\d+)\s+ID\s+(\d+)\s+TAG\s+(\"[^\"]*\"|\S+)/
      "PARREGBEGIN CPU $1 ID " .
      (defined($DefParreg{$2}) ? "\"$DefParreg{$2}\"" :
       ($error = 1, (print STDERR "Undefined parallel region $2: $_"), $2 ) ) .
      " TAG $3" /e ) or
    ( /EXCHEXT/ and /\s(DOWN|UP|)TO\s+(\d+)/ and
      ( defined $DefStates{$2} && defined $DefActs{$DefStatesAct{$2}} ?
	s/\s(DOWN|UP|)TO\s+(\d+)/ $1TO "$DefActs{$DefStatesAct{$2}}:$DefStates{$2}"/g :
	($error = 1, print STDERR ((defined($DefStates{$2}) ? "undefined activity $DefStates{$2}" :
	       "undefined state $2") . ": $_"), (s/\s(DOWN|UP|)TO\s+(\d+)/ $1TO undefined/))) ) or 
    ( /(BEGINSCOPE|ENDSCOPE)/ and /\sSCOPE\s+(\d+)/ and
      ( defined $DefScopes{$1} && defined $DefActs{$DefScopesAct{$1}} ?
	s/\sSCOPE\s+(\d+)/ SCOPE "$DefActs{$DefScopesAct{$1}}:$DefScopes{$1}" ($DefScopesSCLs{$1})/g :
	($error = 1, print STDERR ((defined($DefScopes{$1}) ? "undefined activity $DefScopes{$1}" :
	       "undefined scope $1") . ": $_"), (s/\sSCOPE\s+(\d+)/ SCOPE undefined/))) ) or 
    ( s/\s(SENDMSG|RECVMSG)\s+(\d+)/ $1 COM $2/ ) or
    ( /\sCOLLOP\s+(\w+)\s+(\d+)\s+COM\s+(\d+)/ and
      defined $DefOps{$2} ?
        s/\sCOLLOP\s+(\w+)\s+(\d+)\s+COM\s+(\d+)/ COLLOP $1 $DefOps{$2} COM $3/ :
	($error = 1, print (STDERR "undefined operation $2\n"))) or
    ( /\sFILEIO(BEGIN|END)\s+(\w+)\s+ON\s+CPUID\s+(\d+)\s+(\d+)\s+(\d+)/ and
      defined $DefIO{$4} ?
        s/\sFILEIO(BEGIN|END)\s+(\w+)\s+ON\s+CPUID\s+(\d+)\s+(\d+)\s+(\d+)/ FILEIO$1 $2 ON CPU $3 $DefIO{$4} LENGTH $5/ :
        ($error = 1, print (STDERR "undefined IO token $4\n"))) or
    ( /\sGLOBALOP\s+(\d+)\s+ON\s+(\d+)\s+(\d+)/ and
      defined $DefOps{$1} ?
        s/\sGLOBALOP\s+(\d+)\s+ON\s+(\d+)\s+(\d+)\s+(\d+)\s+(-?\d+)\s+(-?\d+)/ GLOBALOP $DefOps{$1} ON $2 COM $3 ROOT $4 SENT $5 RECVD $6 DURATION/g :
	($error = 1, print (STDERR "undefined operation $1\n")));

    if( /\s+COM\s+(\d+)\b/ ) {
      defined $CommNames{$1} ?
	s/\s+COM\s+(\d+)\b/ COM $CommNames{$1}($1)/g :
	($error = 1, print (STDERR "undefined communicator $1\n") );
    }

    while ( /FUNCTION\s+(\d+)/ ) {
      my $func = $1;
      my $replacement;

      if( defined $DefStates{$func} &&
          defined $DefActs{$DefStatesAct{$func}} ) {
        $replacement = "\"$DefActs{$DefStatesAct{$func}}:$DefStates{$func}\"";
      } else {
        $error = 1;
        print STDERR ((defined($DefStates{$func}) ? "undefined activity $DefStates{$func}" :
                       "undefined state $func") . ": $_");
        $replacement = "undefined";
      }
      s/FUNCTION\s+$func/FUNCTION $replacement/;
    }

    s/ONETOONE(.*?)SCLS\s+(\d+)\s+(\d+)/ONETOONE$1SCL $2 SCL $3/;
    s:COLLOP(.*?) SCLS((\s+\d+)+):"COLLOP" . $1 . join( " SCL ", split( /\s+/, $2 ) ):e;

    /\sSCL\s+(\d+):(-?\d+)/ and
      ( $1 == 0 ? 
        defined $DefFiles{$1} ?s/\sSCL\s+(\d+):(-?\d+)/ SCL "unknown:-1"/g :
        s/\sSCL\s+(\d+):(-?\d+)/ SCL "$DefFiles{$1}:$2"/g :
        ($error = 1, print (STDERR "undefined file $1\n")));
    s/\sSCL\s+268435454/ SCL return/g;
    s/\sSCL\s+268435453/ SCL none/g;
    s/\sSCL\s+-2/ SCL return/g;

    while (/\s(SCLS?)\s+((?:\d+\s+)*)(\d+)/) {
      my ($keyword, $other, $scl) = ($1, $2, $3);
      my $replacement = $DefSrcs{"$scl"};

      if (defined $replacement) {
	while ($replacement =~ /\b(\d+):(-?\d+)/) {
          if ($1 == 0) {
            $replacement =~ s/\b(\d+):(-?\d+)/"unknown:-1"/;
          } elsif (defined $DefFiles{$1}) {
	    $replacement =~ s/\b(\d+):(-?\d+)/"$DefFiles{$1}:$2"/;
	  } else {
	    $error = 1, print (STDERR "undefined file $1\n");
	    $replacement =~ s/\b(\d+):(-?\d+)/"_$1_:$2"/;
	  }
	}
      } else {
        if( $scl ) {
          $error = 1, print (STDERR "undefined SCL $scl\n");
        }
	$replacement = "_${scl}_";
      }

      if ($replacement =~ /^\s*$/) {
	s/\s$keyword\s+$other$scl\b//g;
      }	else {
	s/\s$keyword\s+$other$scl\b/ $keyword $other$replacement/g;
      }
    }
  }

  # replace cryptic decimal number for CPU/GROUP by more readable representation
  my $cpu;
  s/\b(CPU|FROM|TO|BY|ON|PEER|CPUID)\s+(\d+)(\s)/$cpu = $2; "$1 " . TranslateCPUs( $2 ) . $3 /eg;
  if( defined $cpu && defined $DefCPUNames{$cpu} ) {
     s/\n$/ ($DefCPUNames{$cpu})\n/;
  }
  # s/\b(GROUPS|CPUS)(\s+\d+)+/"$1 " . TranslateCPUs( "$2" )/eg;

  # dump (possibly modified) line
  print;
}

exit $error;

