#!/bin/sh
# ------------------------------------------------------------
# Main default options
# ------------------------------------------------------------
COLLECT_IMPI_NATIVE_STATISTICS=true

USE_COLLECTOR=true
    # Collector options
    COLLECT_OMP_STATISTICS=true
    COLLECT_PAPI_COUNRTERS=

# By default if the file name is not set IMPI writes statistics info
# into files: 'stats.txt', 'stats(2)'.txt, 'stats(3)'.txt, etc
CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME=stats.txt

# Note: To disable an option by default use the syntax: 'OPT='

# ------------------------------------------------------------
# Handling command line arguments
# ------------------------------------------------------------
SCRIPT_NAME="mpsvars.sh"
SHOW_HELP_AND_EXIT=
RESET_AND_EXIT=
SHOW_ERROR_IN_OPTIONS_AND_EXIT=
VERBOSE_MODE=
SILENT_MODE=
MPS_TOOL_ROOT=
CUSTOM_PAPI_LIB_DIR=
COLLECT_VTUNE_COUNRTERS=
COLLECT_PAPI_COUNRTERS=
MPS_IS_SUPPORTED=
export MPS_FILE_POSTFIX="_%D-%T"

for option
do
    case $option in

    -h | --help | help )
        SHOW_HELP_AND_EXIT=true
        ;;

    -r | --reset | reset )
        RESET_AND_EXIT=true
        ;;

    -v | --verbose | verbose )
        VERBOSE_MODE=true
        ;;

    -nm | --noimpistats | --noimpistat | noimpistats | noimpistat )
        COLLECT_IMPI_NATIVE_STATISTICS=
        ;;

    -nc | --nocollector | nocollector )
        USE_COLLECTOR=
        ;;

    -p | --papi | papi )
        COLLECT_PAPI_COUNRTERS=true
        ;;

    -w | --vtune | vtune )
        COLLECT_VTUNE_COUNRTERS=true
        ;;

    -no | --noopenmp | --noomp | noopenmp | noomp )
        COLLECT_OMP_STATISTICS=
        ;;

    -pd=* | --papidir=* )
        CUSTOM_PAPI_LIB_DIR=${option#*=}
        ;;

    -td=* | --tooldir=* )
        MPS_TOOL_ROOT=${option#*=}
        ;;

    -s | --silent )
        SILENT_MODE=true
        ;;

    * )
        SHOW_ERROR_IN_OPTIONS_AND_EXIT=$option
        ;; 
    esac
done

if [ $SILENT_MODE ] ; then
    VERBOSE_MODE=
fi

# ------------------------------------------------------------
# Unknown command line option
# ------------------------------------------------------------
if [ $SHOW_ERROR_IN_OPTIONS_AND_EXIT ] && [ ! $SILENT_MODE ]; then
    echo $SCRIPT_NAME: ERROR: Unknown command line option: $SHOW_ERROR_IN_OPTIONS_AND_EXIT >&2
    echo Try \'$SCRIPT_NAME --help\' for more information. >&2
# ------------------------------------------------------------
# Show help
# ------------------------------------------------------------
elif [ $SHOW_HELP_AND_EXIT ] && [ ! $SILENT_MODE ]; then
    echo "Copyright(C) 2014-2015, Intel Corporation. All rights reserved."
    echo
    echo "Usage: $SCRIPT_NAME [options]"
    echo "Where options:"
    echo "  --help, -h        Display this help and exit"
    echo "  --silent, -s      Silent mode"
    echo "  --verbose, -v     Enable verbose script mode"
    echo "  --reset, -r       Clean up environment variables"
    echo "  --noimpistats     Do not collect the Intel(R) MPI Library native statistics"
    echo "  --nocollector     Do not collect MPI Performance Snapshot statistics"
    echo "  --papi, -p        Collect hardware (PAPI) counters statistics (no Intel(R) Many Integrated Core Architecture support)"
    echo "  --vtune, -w       Collect Performance Metrics by means of Intel(R) VTune(TM) Amplifier XE tool. Use either --vtune or --papi."
    echo "  --noopenmp        Do not collect OpenMP statistics"
    echo "  --papidir=<dir>   Setup own directory with PAPI libraries"
    echo ""
# ------------------------------------------------------------
# Reset environment variables
# ------------------------------------------------------------
elif [ $RESET_AND_EXIT ]; then
    if [ $VERBOSE_MODE ]; then
        echo "Cleaning variables: \$MPS_TOOL_ROOT, \$MPS_I_MPI_STATS, \$MPS_I_MPI_STATS_FILE, \$MPS_INTEL_LIBITTNOTIFY64, \$MPS_KMP_FORKJOIN_FRAMES_MODE, \$MPS_LD_PRELOAD, \$MPS_VTUNE_PATH, \$MPS_VTUNE_RESULTS_PATH, \$AMPLXE_EXPERIMENTAL"
    fi

    unset MPS_TOOL_ROOT
    unset MPS_I_MPI_STATS
    unset MPS_I_MPI_STATS_FILE
    unset MPS_INTEL_LIBITTNOTIFY64
    unset MPS_KMP_FORKJOIN_FRAMES_MODE
    unset MPS_LD_PRELOAD
    unset MPS_VTUNE_PATH
    unset MPS_VTUNE_RESULTS_PATH
    unset AMPLXE_EXPERIMENTAL
else
# ------------------------------------------------------------
# MAIN block
# ------------------------------------------------------------

    # ------------------------------------------------------------
    # Setup default paths to the tool license - $INTEL_LICENSE_FILE
    # ------------------------------------------------------------
    if [ -n "${INTEL_LICENSE_FILE}" ]; then
        INTEL_LICENSE_FILE="/opt/intel/licenses:/root/intel/licenses:${INTEL_LICENSE_FILE}"
    else
        INTEL_LICENSE_FILE="/opt/intel/licenses:/root/intel/licenses"
    fi

    # ------------------------------------------------------------
    # Clean-up exporting variabes
    # ------------------------------------------------------------
    unset MPS_I_MPI_STATS
    unset MPS_I_MPI_STATS_FILE
    unset MPS_INTEL_LIBITTNOTIFY64
    unset MPS_KMP_FORKJOIN_FRAMES_MODE
    unset MPS_LD_PRELOAD
    unset MPS_VTUNE_PATH
    unset MPS_VTUNE_RESULTS_PATH
    unset AMPLXE_EXPERIMENTAL

    # ------------------------------------------------------------
    # Detect and check this script dir name - $MPS_TOOL_ROOT
    # ------------------------------------------------------------
    folderIsValid()
    {
        folder=$1; shift
        if [ -f $folder/intel64/bin/$SCRIPT_NAME ]; then
            return 0
        else
            return 1
        fi
    }

    # Check the manually set folder
    if [ $MPS_TOOL_ROOT ]; then
        if ! $( folderIsValid $MPS_TOOL_ROOT ); then
            if [ ! $SILENT_MODE ]; then
                echo $SCRIPT_NAME: WARNING Could not find $SCRIPT_NAME in the \'$MPS_TOOL_ROOT/intel64/bin\' >&2
                echo Trying to auto-detect... >&2
            fi
            MPS_TOOL_ROOT=
        fi 
    fi

    # Try to auto-detect
    if [ -z $MPS_TOOL_ROOT ]; then
        if [ $BASH_SOURCE ]; then
           AUTO_DETECTED_THIS_DIR=$( cd $( dirname "$BASH_SOURCE" ) && pwd -P )
        elif [ ${BASH_SOURCE[0]} ]; then
           AUTO_DETECTED_THIS_DIR=$( cd $( dirname "${BASH_SOURCE[0]}" ) && pwd -P )
        else
           AUTO_DETECTED_THIS_DIR=$( cd $( dirname "$0" ) && pwd -P )
        fi

        AUTO_DETECTED_THIS_DIR=$( cd $AUTO_DETECTED_THIS_DIR/../.. && pwd -P )
        if $( folderIsValid $AUTO_DETECTED_THIS_DIR ); then
            MPS_TOOL_ROOT=$AUTO_DETECTED_THIS_DIR
        fi 
    fi

    if [ -z $MPS_TOOL_ROOT ]; then
        if [ ! $SILENT_MODE ]; then
            echo $SCRIPT_NAME: ERROR: Could not find correct folders structure >&2
        fi 
    else
        if [ $VERBOSE_MODE ]; then
            echo "Setting tool environment for the folder: $MPS_TOOL_ROOT"
        fi

        # ------------------------------------------------------------
        # WORK block
        # ------------------------------------------------------------

        # ------------------------------------------------------------
        # Setup IMPI native statistics collecting
        # ------------------------------------------------------------
        if [ $COLLECT_IMPI_NATIVE_STATISTICS ]; then
            MPS_I_MPI_STATS=20

            if [ $CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME ]; then
                MPS_I_MPI_STATS_FILE=$CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME
            fi
        fi

        # ------------------------------------------------------------
        # Setup the collector
        # ------------------------------------------------------------
        if [ $USE_COLLECTOR ]; then
            MPS_LIB_DIR=$MPS_TOOL_ROOT/intel64/slib
            MPS_LIB_DIR_MIC=$MPS_TOOL_ROOT/mic/slib

            # Paths to the libraries:
            #     $MPS_LIB_DIR (or $CUSTOM_PAPI_LIB_DIR)
            #                         - for PAPI libs: libpfm.so, libpapi.so
            #     $MPS_LIB_DIR  - for new OpenMP lib: libiomp5.so
            #     $MPS_LIB_DIR  - for collector libs: libmps.so (or libmps_nopapi.so)
            if [ $LD_LIBRARY_PATH ]; then
                LD_LIBRARY_PATH=$MPS_LIB_DIR:$LD_LIBRARY_PATH
            else
                LD_LIBRARY_PATH=$MPS_LIB_DIR
            fi
            LD_LIBRARY_PATH=$MPS_LIB_DIR_MIC:$LD_LIBRARY_PATH

            if [ ! $COLLECT_PAPI_COUNRTERS ] && [ ! $COLLECT_VTUNE_COUNRTERS ]; then
                if [ ! $SILENT_MODE ]; then
                    echo $SCRIPT_NAME: "Warning: Hardware events collection is disabled by default. To enable it, run mpsvars.sh with --vtune (recommended) or --papi option."
                fi
            fi

            if [ $COLLECT_PAPI_COUNRTERS ] && [ $COLLECT_VTUNE_COUNRTERS ]; then
                if [ ! $SILENT_MODE ]; then
                    echo $SCRIPT_NAME: "Error: --papi and --vtune options cannot be used together. Options are ignored."
                fi
            fi

            # Collector option 1: PAPI
            if [ $COLLECT_PAPI_COUNRTERS ] && [ ! $COLLECT_VTUNE_COUNRTERS ]; then
                STAT_TOOL_LIB_NAME=libmps.so
                if [ ! $SILENT_MODE ]; then
                    echo $SCRIPT_NAME: "Warning: PAPI library can be used for hosts only."
                fi

                if [ $CUSTOM_PAPI_LIB_DIR ]; then
                    LD_LIBRARY_PATH=$CUSTOM_PAPI_LIB_DIR:$LD_LIBRARY_PATH
                fi
            else
                STAT_TOOL_LIB_NAME=libmps_nopapi.so
            fi

            # Collector option 2: VTune
            if [ ! $COLLECT_PAPI_COUNRTERS ] && [ $COLLECT_VTUNE_COUNRTERS ]; then
                MPS_ENV=`env | grep VTUNE_AMPLIFIER_XE`
                if [ "x$MPS_ENV" != "x" ]; then
                    MPS_VTUNE_PATH=`which amplxe-cl 2>/dev/null`
                    if [ "x$MPS_VTUNE_PATH" != "x" ]; then
                        MPS_VTUNE_VER=`amplxe-cl -V | grep "Amplifier XE 2016 Beta"`
                        if [ "x$MPS_VTUNE_VER" != "x" ]; then
                            unset MPS_VTUNE_PATH
                            if [ ! $SILENT_MODE ]; then
                                echo $SCRIPT_NAME: "Error: Current version of Intel(R) VTune(TM) Amplifier XE doesn't support MPI Performance Snapshot utility."
                            fi
                        else
                            MPS_IS_SUPPORTED=true
                        fi
                    fi
                    if [ "x$MPS_VTUNE_PATH" != "x" ]; then
                        MPS_SUPPORT=`AMPLXE_EXPERIMENTAL=mps amplxe-cl -collect-list | grep mps`
                        if [ "x$MPS_SUPPORT" != "x" ]; then
                            MPS_VTUNE_RESULTS_PATH=_mps/results
                            AMPLXE_EXPERIMENTAL=mps
                        else
                            if [ ! $SILENT_MODE ]; then
                                echo $SCRIPT_NAME: "Error: Current version of Intel(R) VTune(TM) Amplifier XE doesn't support MPI Performance Snapshot utility."
                            fi
                        fi
                    else
                        if [ ! $SILENT_MODE ] && [ $MPS_IS_SUPPORTED ]; then
                            echo $SCRIPT_NAME: "Error: Could not find Intel(R) VTune(TM) Amplifier XE (amplxe-cl)"
                        fi
                    fi
                else
                    if [ ! $SILENT_MODE ]; then
                        echo $SCRIPT_NAME: "Error: Could not find Intel(R) VTune(TM) Amplifier XE environment. Please source amplxe-vars.sh"
                    fi
                fi
            fi

            # For IMPI library instrumentation & memory & HW (PAPI) counters profiling:
            MPS_LD_PRELOAD=$STAT_TOOL_LIB_NAME

            # Collector option 3: OpenMP
            if [ $COLLECT_OMP_STATISTICS ]; then
                # For OpenMP library instrumentation
                MPS_INTEL_LIBITTNOTIFY64=$STAT_TOOL_LIB_NAME
                MPS_KMP_FORKJOIN_FRAMES_MODE=3

                # Forestall OpenMP library loading (before default compiler OpenMP library)
#                LD_PRELOAD=$MPS_LIB_DIR/libiomp5.so:$LD_PRELOAD
            fi
        fi

        # ------------------------------------------------------------
        # Setup the analyser tool MPI Performance Snapshot: mps
        # ------------------------------------------------------------
        PATH=$MPS_TOOL_ROOT/intel64/bin:$PATH

        # ------------------------------------------------------------
        # Export section
        # ------------------------------------------------------------
        export INTEL_LICENSE_FILE
        export PATH
        export LD_LIBRARY_PATH

#        test $MPS_LIB_DIR                  && export MPS_LIB_DIR
#        test $MPS_LIB_DIR_MIC              && export MPS_LIB_DIR_MIC

        test $MPS_TOOL_ROOT                 && export MPS_TOOL_ROOT
        test $MPS_LD_PRELOAD                && export MPS_LD_PRELOAD

        test $MPS_I_MPI_STATS               && export MPS_I_MPI_STATS
        test $MPS_I_MPI_STATS_FILE          && export MPS_I_MPI_STATS_FILE

        test $MPS_INTEL_LIBITTNOTIFY64      && export MPS_INTEL_LIBITTNOTIFY64
        test $MPS_KMP_FORKJOIN_FRAMES_MODE  && export MPS_KMP_FORKJOIN_FRAMES_MODE

        test MPS_VTUNE_PATH                 && export MPS_VTUNE_PATH
        test MPS_VTUNE_RESULTS_PATH         && export MPS_VTUNE_RESULTS_PATH
        test AMPLXE_EXPERIMENTAL            && export AMPLXE_EXPERIMENTAL

        # ------------------------------------------------------------
        # Finally print variables
        # ------------------------------------------------------------
        if [ $VERBOSE_MODE ]; then
            echo "    \$INTEL_LICENSE_FILE           = $INTEL_LICENSE_FILE"
            echo "    \$LD_LIBRARY_PATH              = $LD_LIBRARY_PATH"
            echo ""
            echo "    \$MPS_TOOL_ROOT                = $MPS_TOOL_ROOT"
            echo "    \$MPS_LD_PRELOAD               = $MPS_LD_PRELOAD"
            echo "    \$MPS_I_MPI_STATS              = $MPS_I_MPI_STATS"
            echo "    \$MPS_I_MPI_STATS_FILE         = $MPS_I_MPI_STATS_FILE"
            echo "    \$MPS_INTEL_LIBITTNOTIFY64     = $MPS_INTEL_LIBITTNOTIFY64"
            echo "    \$MPS_KMP_FORKJOIN_FRAMES_MODE = $MPS_KMP_FORKJOIN_FRAMES_MODE"
            echo "    \$OMP_NUM_THREADS              = $OMP_NUM_THREADS"
            echo "    \$MPS_VTUNE_PATH               = $MPS_VTUNE_PATH"
            echo "    \$MPS_VTUNE_RESULTS_PATH       = $MPS_VTUNE_RESULTS_PATH"
            echo "    \$AMPLXE_EXPERIMENTAL          = $AMPLXE_EXPERIMENTAL"
        fi

    fi # WORK block
fi # MAIN block
