/***************************************************************************************
 *
 * Intel(R) Trace Collector Library.
 *
 * Copyright(C) 2003-2014 Intel Corporation.
 * All Rights Reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material contains trade secrets and proprietary and confidential
 * information of Intel or its suppliers and licensors. The Material
 * is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 * 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 * 
 ***************************************************************************************/

/**
 *
 * @name PAPI counter tracing on top of public VT API
 *
 **/
/*@{ */

/**
 * @file
 **/
/***************************************************************************************/

#include "config.h"

#include <VT.h>
#include <stdio.h>
#include <string.h>

#include <papi.h>

#include <sys/resource.h>
#include <unistd.h>


/** an array with all enabled counters (used as cache for callback) */
static UINT32 VT_handles[4];

static unsigned int VT_SampleCounters( void *custom, double ts, void *mem )
{
    int i;
    INT64 *sample = mem;

    UINT32 *handle;

    /*Get the new PAPI data */
    float real_time, proc_time, mflops;
    long long flpins;

    int retval;
    retval=PAPI_flops( &real_time, &proc_time, &flpins, &mflops);
    if(retval<PAPI_OK)
        printf("PAPI error: %s\n", PAPI_strerror(retval));

    /* copy the new PAPI samples */
    *sample = real_time;
    sample++;
    
    *sample = proc_time;
    sample++;
    
    *sample = flpins;
    sample++;
    
    *sample = mflops;
    sample++;
    
    /* Copy counter handle*/
    handle = (UINT32 *)sample;
    for( i = 0; i < 4; i++ ) {
        *handle = VT_handles[i];
        handle++;
    }
                                                                                    

    return 4;
}

/**
 * This is called when VT is ready to finalize logging.
 */
static void VT_FinalizeCallback( void )
{
    /* stop and disable logging of PAPI */
    PAPI_shutdown();
}

/**
 * This is called when VT has initialized.
 */
static void VT_InitCallback( void )
{
    static const UINT64 ibounds[2] = { 0, (((UINT64)0xFFFFFFFF)<<32) | 0xFFFFFFFF };
    static const FLOAT64 fbounds[2] = { 0, 1e30 };
    
    float real_time, proc_time, mflops;
    long long flpins;

    /* Define one counter */
    int class, counterhandle;

    VT_classdef( "PAPI", &class );

    VT_countdef( "Real time", class,
		VT_COUNT_VALID_SAMPLE|VT_COUNT_ABSVAL|VT_COUNT_FLOAT, VT_GROUP_PROCESS,
		fbounds, "#",
		&counterhandle );
    VT_handles[0] = counterhandle;

    VT_countdef( "Proc time", class,
		VT_COUNT_VALID_SAMPLE|VT_COUNT_ABSVAL|VT_COUNT_FLOAT, VT_GROUP_PROCESS,
		fbounds, "#",
		&counterhandle );
    VT_handles[1] = counterhandle;

    VT_countdef( "Flpins", class,
		VT_COUNT_VALID_SAMPLE|VT_COUNT_ABSVAL|VT_COUNT_INTEGER64, VT_GROUP_PROCESS,
		ibounds, "#",
		&counterhandle );
    VT_handles[2] = counterhandle;

    VT_countdef( "MFlops", class,
		VT_COUNT_VALID_SAMPLE|VT_COUNT_ABSVAL|VT_COUNT_FLOAT, VT_GROUP_PROCESS,
		fbounds, "#",
		&counterhandle );
    VT_handles[3] = counterhandle;

    /* we should give PAPI a chance to free resources when VT exits */
    VT_setfinalizecallback( VT_FinalizeCallback );

    /*Start High Level PAPI collecting */
    int retval;
    retval=PAPI_flops( &real_time, &proc_time, &flpins, &mflops);
    if(retval<PAPI_OK)
        printf("PAPI error: %s\n", PAPI_strerror(retval));

    VT_countsetcallback( VT_SampleCounters, 0, 4 );
}

/**
 * We overwrite the corresponding VT variable with a pointer
 * to our init function.
 */
VT_Callback_t VT_APP_initcallback = VT_InitCallback;

/*@}*/