#
# ------------------------------------------------------------
# Main default options
# ------------------------------------------------------------
set COLLECT_IMPI_NATIVE_STATISTICS=1

set USE_COLLECTOR = 1

    # Collector options
set    COLLECT_OMP_STATISTICS = 1
set    COLLECT_PAPI_COUNRTERS = 1

# By default if the file name is not set IMPI writes statistics info
# into files: 'stats.txt', 'stats(2)'.txt, 'stats(3)'.txt, etc
set CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME = "stats.txt"

# Note: To disable an option by default use the syntax: 'OPT='

# ------------------------------------------------------------
# Handling command line arguments
# ------------------------------------------------------------
set SCRIPT_NAME = "mpsvars.csh"
set SHOW_HELP_AND_EXIT = 0
set RESET_AND_EXIT = 0
set SHOW_ERROR_IN_OPTIONS_AND_EXIT = ""
set VERBOSE_MODE = 0
set SILENT_MODE = 0
set MPS_TOOL_ROOT = ""
set CUSTOM_PAPI_LIB_DIR = ""
set COLLECT_VTUNE_COUNRTERS = 0
set COLLECT_PAPI_COUNRTERS = 0
setenv MPS_FILE_POSTFIX="_%D-%T"

set i = 1
while ( $i <= $#argv )
    set tmp_var = "$argv[$i]"
    if ( "${tmp_var}" == "-h" || "${tmp_var}" == "--help" || "${tmp_var}" == "help" ) then
        @ SHOW_HELP_AND_EXIT = 1
    else if ("${tmp_var}" == "-r" || "${tmp_var}" == "--reset" || "${tmp_var}" == "reset" ) then
        @ RESET_AND_EXIT = 1
    else if ( "${tmp_var}" == "-v" || "${tmp_var}" == "--verbose" || "${tmp_var}" == "verbose" ) then
        @ VERBOSE_MODE = 1
    else if ( "${tmp_var}" == "-nm" || "${tmp_var}" == "--noimpistats" || "${tmp_var}" == "--noimpistat" || "${tmp_var}" == "noimpistats" || "${tmp_var}" == "noimpistat" ) then
        @ COLLECT_IMPI_NATIVE_STATISTICS = 0
    else if ( "${tmp_var}" == "-nc" || "${tmp_var}" == "--nocollector" || "${tmp_var}" == "nocollector" ) then
        @ USE_COLLECTOR = 0
    else if ( "${tmp_var}" == "-p" || "${tmp_var}" == "--papi" || "${tmp_var}" == "papi" ) then
        @ COLLECT_PAPI_COUNRTERS = 1
    else if ( "${tmp_var}" == "-w" || "${tmp_var}" == "--vtune" || "${tmp_var}" == "vtune" ) then
        @ COLLECT_VTUNE_COUNRTERS = 1
    else if ( "${tmp_var}" == "-no" || "${tmp_var}" == "--noopenmp" || "${tmp_var}" == "--noomp" || "${tmp_var}" == "noopenmp" || "${tmp_var}" == "noomp" ) then
        @ COLLECT_OMP_STATISTICS = 0
    else if ( "${tmp_var}" == "-pd" || "${tmp_var}" == "--papidir" ) then
        @ i++
        if ( $i <= $#argv ) then
            set CUSTOM_PAPI_LIB_DIR = "${argv[$i]}"
            #check correctness of the path
        else
            if ( ${SILENT_MODE} == 0 ) then
                echo "${SCRIPT_NAME}: ERROR: There is no directory name for --papidir option"
            endif
            exit 1
        endif
    else if ( "${argv[$i]}" == "-td" || "${tmp_var}" == "--tooldir" ) then
        @ i++
        if ( $i <= $#argv ) then
            set MPS_TOOL_ROOT = "${argv[$i]}"
            # we will try to autodetect... or mitgh be check correctness of the the path
        else
            if ( ${SILENT_MODE} == 0 ) then
                echo "${SCRIPT_NAME}: ERROR: There is no directory name for --tooldir option"
            endif
            exit 1
        endif
    else if ( "${tmp_var}" == "-s" || "${tmp_var}" == "--silent" ) then
        @ SILENT_MODE = 1
    else
        set SHOW_ERROR_IN_OPTIONS_AND_EXIT = "${tmp_var}"
    endif
    @ i++
end

if ( ${SILENT_MODE} == 1 ) then
    @ VERBOSE_MODE = 0
endif

# ------------------------------------------------------------
# Unknown command line option
# ------------------------------------------------------------
if ( "${SHOW_ERROR_IN_OPTIONS_AND_EXIT}" != ""  && ${SILENT_MODE} == 0 ) then
    echo "${SCRIPT_NAME}: ERROR: Unknown command line option: ${SHOW_ERROR_IN_OPTIONS_AND_EXIT}"
    echo "Try '${SCRIPT_NAME} --help' for more information."
# ------------------------------------------------------------
# Show help
# ------------------------------------------------------------
else if ( ${SHOW_HELP_AND_EXIT} == 1 && ${SILENT_MODE} == 0 ) then
    echo "Copyright(C) 2014-2015, Intel Corporation. All rights reserved."
    echo
    echo "Usage: $SCRIPT_NAME [options]"
    echo "Where options:"
    echo "  --help, -h        Display this help and exit"
    echo "  --silent, -s      Silent mode"
    echo "  --verbose, -v     Enable verbose script mode"
    echo "  --reset, -r       Clean up environment variables"
    echo "  --noimpistats     Do not collect the Intel(R) MPI Library native statistics"
    echo "  --nocollector     Do not collect MPI Performance Snapshot statistics"
    echo "  --papi, -p        Collect hardware (PAPI) counters statistics (no Intel(R) Many Integrated Core Architecture support)"
    echo "  --vtune, -w       Collect Performance Metrics by means of Intel(R) VTune(TM) Amplifier XE tool. Use either --vtune or --papi."
    echo "  --noopenmp        Do not collect OpenMP statistics"
    echo "  --papidir <dir>   Setup own directory with PAPI libraries"
    echo ""
# ------------------------------------------------------------
# Reset environment variables
# ------------------------------------------------------------
else if ( ${RESET_AND_EXIT} == 1 ) then
    if ( ${VERBOSE_MODE} == 1 ) then
        echo "Cleaning variables: MPS_TOOL_ROOT, MPS_I_MPI_STATS, MPS_I_MPI_STATS_FILE, MPS_INTEL_LIBITTNOTIFY64, MPS_KMP_FORKJOIN_FRAMES_MODE, MPS_LD_PRELOAD, MPS_VTUNE_PATH, MPS_VTUNE_RESULTS_PATH, AMPLXE_EXPERIMENTAL"
    endif

    unsetenv MPS_TOOL_ROOT
    unsetenv MPS_I_MPI_STATS
    unsetenv MPS_I_MPI_STATS_FILE
    unsetenv MPS_INTEL_LIBITTNOTIFY64
    unsetenv MPS_KMP_FORKJOIN_FRAMES_MODE
    unsetenv MPS_LD_PRELOAD
    unsetenv MPS_VTUNE_PATH
    unsetenv MPS_VTUNE_RESULTS_PATH
    unsetenv AMPLXE_EXPERIMENTAL

else
# ------------------------------------------------------------
# MAIN block
# ------------------------------------------------------------

    # ------------------------------------------------------------
    # Setup default paths to the tool license - $INTEL_LICENSE_FILE
    # ------------------------------------------------------------
    if $?INTEL_LICENSE_FILE then
        setenv INTEL_LICENSE_FILE "/opt/intel/licenses:/root/intel/licenses:${INTEL_LICENSE_FILE}"
    else
        setenv INTEL_LICENSE_FILE "/opt/intel/licenses:/root/intel/licenses"
    endif

    # ------------------------------------------------------------
    # Clean-up exporting variabes
    # ------------------------------------------------------------
    unsetenv MPS_I_MPI_STATS
    unsetenv MPS_I_MPI_STATS_FILE
    unsetenv MPS_INTEL_LIBITTNOTIFY64
    unsetenv MPS_KMP_FORKJOIN_FRAMES_MODE
    unsetenv MPS_LD_PRELOAD
    unsetenv MPS_VTUNE_PATH
    unsetenv MPS_VTUNE_RESULTS_PATH
    unsetenv AMPLXE_EXPERIMENTAL

    # Check the manually set folder
    if ( "${MPS_TOOL_ROOT}" != "" ) then
        if ( ! -f ${MPS_TOOL_ROOT}/intel64/bin/${SCRIPT_NAME} ) then
            if ( ${SILENT_MODE} == 0 ) then
                echo "${SCRIPT_NAME}: WARNING: Could not find ${SCRIPT_NAME} in the '${MPS_TOOL_ROOT}/intel64/bin' folder"
                echo "Trying to auto-detect..."
            endif
            set MPS_TOOL_ROOT = ""
        endif
    endif

    # Try to auto-detect
    if ( "${MPS_TOOL_ROOT}" == "" ) then
        set dir = `readlink -f $0`
        set dir = `dirname $dir`
        set AUTO_DETECTED_DIR = `cd $dir/../.. && pwd`
        if ( -f ${AUTO_DETECTED_DIR}/intel64/bin/${SCRIPT_NAME} ) then
            set MPS_TOOL_ROOT = "${AUTO_DETECTED_DIR}"
        endif
    endif

    if ( "${MPS_TOOL_ROOT}" == "" ) then
        echo "END"
        if ( ${SILENT_MODE} == 0 ) then
            echo "${SCRIPT_NAME}: ERROR: Could not find correct folders structure."
        endif
    else
        if ( ${VERBOSE_MODE} == 1 ) then
            echo "Setting tool environment for the folder: ${MPS_TOOL_ROOT}"
        endif

        # ------------------------------------------------------------
        # WORK block
        # ------------------------------------------------------------

        # ------------------------------------------------------------
        # Setup IMPI native statistics collecting
        # ------------------------------------------------------------
        if ( ${COLLECT_IMPI_NATIVE_STATISTICS} == 1 ) then
            setenv MPS_I_MPI_STATS 20

            if ( "${CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME}" != "" ) then
                setenv MPS_I_MPI_STATS_FILE "$CUSTOM_IMPI_NATIVE_STATISTICS_FILE_NAME"
            endif
        endif

        # ------------------------------------------------------------
        # Setup the collector
        # ------------------------------------------------------------
        if ( ${USE_COLLECTOR} == 1 ) then
            set MPS_LIB_DIR = "${MPS_TOOL_ROOT}/intel64/slib"
            set MPS_LIB_DIR_MIC = "${MPS_TOOL_ROOT}/mic/slib"

            # Paths to the libraries:
            #     $MPS_LIB_DIR (or $CUSTOM_PAPI_LIB_DIR)
            #                         - for PAPI libs: libpfm.so, libpapi.so
            #     $MPS_LIB_DIR  - for new OpenMP lib: libiomp5.so
            #     $MPS_LIB_DIR  - for collector libs: libmps.so (or libmps_nopapi.so)
            if $?LD_LIBRARY_PATH then
                setenv LD_LIBRARY_PATH ${MPS_LIB_DIR}:${LD_LIBRARY_PATH}
            else
                setenv LD_LIBRARY_PATH ${MPS_LIB_DIR}
            endif
            setenv  LD_LIBRARY_PATH ${MPS_LIB_DIR_MIC}:${LD_LIBRARY_PATH}

            if ( ${COLLECT_PAPI_COUNRTERS} == 0  && ${COLLECT_VTUNE_COUNRTERS} == 0 ) then
                if ( ${SILENT_MODE} == 0 ) then
                    echo "Warning: Hardware events collection is disabled by default. To enable it, run mpsvars.sh with --vtune (recommended) or --papi option."
                endif
            endif

            if ( ${COLLECT_PAPI_COUNRTERS} == 1  && ${COLLECT_VTUNE_COUNRTERS} == 1 ) then
                if ( ${SILENT_MODE} == 0 ) then
                    echo "Error: --papi and --vtune options cannot be used together. Options are ignored."
                endif
            endif

            # Collector option 1: PAPI
            if ( ${COLLECT_PAPI_COUNRTERS} == 1 && ${COLLECT_VTUNE_COUNRTERS} == 0 ) then
                set STAT_TOOL_LIB_NAME = "libmps.so"
                if ( ${SILENT_MODE} == 0 ) then
                    echo "Warning: PAPI library can be used for hosts only."
                endif

                if ( "${CUSTOM_PAPI_LIB_DIR}" != "" ) then
                    setenv LD_LIBRARY_PATH ${CUSTOM_PAPI_LIB_DIR}:${LD_LIBRARY_PATH}
                endif
            else
                set STAT_TOOL_LIB_NAME = "libmps_nopapi.so"
            endif

            # Collector option 2: VTune
            if ( ${COLLECT_PAPI_COUNRTERS} == 0 && ${COLLECT_VTUNE_COUNRTERS} == 1 ) then
                set MPS_VTUNE_PATH = `env | grep VTUNE_AMPLIFIER_XE`
                set MPS_IS_SUPPORTED = 1
                if ( "${MPS_VTUNE_PATH}" != "" ) then
                    set MPS_VTUNE_PATH = `(which amplxe-cl >/tmp/mps_vtune_temp_output.txt) >&/dev/null`
                    set MPS_VTUNE_PATH = `grep -v could /tmp/mps_vtune_temp_output.txt`
                    set remove = `rm -rf /tmp/mps_vtune_temp_output.txt`
                    echo "VTUNE ${MPS_VTUNE_PATH}"
                    if ( "${MPS_VTUNE_PATH}" != "" ) then
                        set MPS_VTUNE_VER = `amplxe-cl -V | grep "Amplifier XE 2016 Beta"`
                        if ( "${MPS_VTUNE_VER}" != "" ) then
                            set MPS_IS_SUPPORTED = 0
                            set MPS_VTUNE_PATH = ""
                            if ( ${SILENT_MODE} == 0 ) then
                                echo "Error: Current version of Intel(R) VTune(TM) Amplifier XE tool doesn't support MPI Performance Snapshot utility."
                            endif
                        endif
                    endif
                    if ( "${MPS_VTUNE_PATH}" != "" ) then
                        setenv AMPLXE_EXPERIMENTAL mps
                        set MPS_SUPPORT = `amplxe-cl -collect-list | grep mps`
                        if ( "${MPS_SUPPORT}" != "" ) then
                            setenv MPS_VTUNE_PATH ${MPS_VTUNE_PATH}
                            setenv MPS_VTUNE_RESULTS_PATH "_mps/results"
                            setenv AMPLXE_EXPERIMENTAL "mps"
                        else
                            unsetenv AMPLXE_EXPERIMENTAL
                            if ( ${SILENT_MODE} == 0 ) then
                                echo "Error: Current version of Intel(R) VTune(TM) Amplifier XE tool doesn't support MPI Performance Snapshot utility."
                            endif
                        endif
                    else
                        if ( ${SILENT_MODE} == 0 && ${MPS_IS_SUPPORTED} == 1 ) then
                            echo "Error: Could not find Intel(R) VTune(TM) Amplifier XE tool (amplxe-cl)"
                        endif
                    endif
                else
                    if ( ${SILENT_MODE} == 0) then
                        echo "Error: Could not find Intel(R) VTune(TM) Amplifier XE tool environment. Please source amplxe-vars.sh"
                    endif
                endif
            endif

            # For IMPI library instrumentation & memory & HW (PAPI) counters profiling:
            setenv MPS_LD_PRELOAD "${STAT_TOOL_LIB_NAME}"

            # Collector option 3: OpenMP
            if ( ${COLLECT_OMP_STATISTICS} == 1 ) then
                # For OpenMP library instrumentation
                setenv MPS_INTEL_LIBITTNOTIFY64 "${STAT_TOOL_LIB_NAME}"
                setenv MPS_KMP_FORKJOIN_FRAMES_MODE 3
            endif
        endif

        # ------------------------------------------------------------
        # Setup the analyser tool MPI Performance Snapshot: mps
        # ------------------------------------------------------------
        setenv  PATH ${MPS_TOOL_ROOT}/intel64/bin:${PATH}
        setenv  MPS_TOOL_ROOT $MPS_TOOL_ROOT

        # ------------------------------------------------------------
        # Finally print variables
        # ------------------------------------------------------------
        if ( ${VERBOSE_MODE} == 1 ) then
            if $?INTEL_LICENSE_FILE then
                echo "    INTEL_LICENSE_FILE           = $INTEL_LICENSE_FILE"
            endif
            if $?LD_LIBRARY_PATH then 
                echo "    LD_LIBRARY_PATH              = $LD_LIBRARY_PATH"
            endif
            echo ""
            if $?MPS_TOOL_ROOT then
                echo "    MPS_TOOL_ROOT                = $MPS_TOOL_ROOT"
            endif
            if $?MPS_LD_PRELOAD then
                echo "    MPS_LD_PRELOAD               = $MPS_LD_PRELOAD"
            endif
            if $?MPS_I_MPI_STATS then
                echo "    MPS_I_MPI_STATS              = $MPS_I_MPI_STATS"
            endif
            if $?MPS_I_MPI_STATS_FILE then
                echo "    MPS_I_MPI_STATS_FILE         = $MPS_I_MPI_STATS_FILE"
            endif
            if $?MPS_INTEL_LIBITTNOTIFY64 then
                echo "    MPS_INTEL_LIBITTNOTIFY64     = $MPS_INTEL_LIBITTNOTIFY64"
            endif
            if $?MPS_KMP_FORKJOIN_FRAMES_MODE then
                echo "    MPS_KMP_FORKJOIN_FRAMES_MODE = $MPS_KMP_FORKJOIN_FRAMES_MODE"
            endif
            if $?MPS_VTUNE_PATH then
                echo "    MPS_VTUNE_PATH               = $MPS_VTUNE_PATH"
            endif
            if $?MPS_VTUNE_RESULTS_PATH then
                echo "    MPS_VTUNE_RESULTS_PATH       = $MPS_VTUNE_RESULTS_PATH"
            endif
            if $?AMPLXE_EXPERIMENTAL then
                echo "    AMPLXE_EXPERIMENTAL          = $AMPLXE_EXPERIMENTAL"
            endif
        endif

    endif # WORK block
endif # MAIN block
